#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "SDL.h"
#include "sdlvf.h"

#define usage() fprintf(stderr, "Usage: %s [-o outfile] [-s start_time] music.ogg\n", argv[0])


int main(int argc, char **argv) {
	int i, done = 0;
	int time_offset = 0;
	unsigned long start_time;
	const char *music_file = 0;
	FILE *sync_out = stdout;
	SDL_Surface *fb;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(argv[i][1] == 'o' && !argv[i][2]) {
				if(!(sync_out = fopen(argv[++i], "w"))) {
					fprintf(stderr, "could not open file \"%s\" for sync data output\n", argv[i]);
					return -1;
				}
			} else if(argv[i][1] == 's' && !argv[i][2]) {
				if(!isdigit(argv[++i][0])) {
					fprintf(stderr, "-s must be followed by a positive integer\n");
					return -1;
				}
				time_offset = atoi(argv[i]);
			} else if(argv[i][1] == 'h' && !argv[i][2]) {
				usage();
				return 0;
			} else {
				fprintf(stderr, "unknown option: %s\n", argv[i]);
				return -1;
			}
		} else {
			if(!music_file) {
				music_file = argv[i];
			} else {
				usage();
				return -1;
			}
		}
	}

	if(!music_file) {
		fprintf(stderr, "no music file specified\n");
		usage();
		return -1;
	}

	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER);
	fb = SDL_SetVideoMode(250, 100, 32, SDL_HWSURFACE);
	SDL_WM_SetCaption("Dummy window to get events", 0);
	SDL_FillRect(fb, 0, 0x00ff0000);
	SDL_Flip(fb);

	sdlvf_init(music_file);

	start_time = SDL_GetTicks() - time_offset;

	while(!done) {
		int sdlvf_error;
		SDL_Event event;

		while(SDL_PollEvent(&event)) {
			if(event.type == SDL_KEYDOWN) {
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
				case 'q':
					done = 1;
					break;

				case SDLK_SPACE:
					{
						static int mark;
						unsigned long time = SDL_GetTicks() - start_time;

						SDL_FillRect(fb, 0, 0x00ffffff);
						SDL_Flip(fb);

						fprintf(sync_out, "mark %d: %lu\n", mark++, time);

						while((SDL_GetTicks() - start_time) - time < 20);
						SDL_FillRect(fb, 0, 0x00ff0000);
						SDL_Flip(fb);
					}
					break;

				default:
					break;
				}
			}
		}

		if((sdlvf_error = sdlvf_check()) != SDLVF_PLAYING) {
			fprintf(stderr, "quit: %s", sdlvf_strerror(sdlvf_error));
			done = 1;
		}
	}

	sdlvf_done();
	if(sync_out != stdout) {
		fclose(sync_out);
	}
	SDL_Quit();
	return 0;
}
